#!/usr/bin/perl -w

#define column number indices

use constant COUNT_INDEX => 0;
use constant BYTES_INDEX => 1;
use constant AVG_INDEX => 2;
use constant OBJ_INDEX => 3;
use constant TYPE_INDEX => 4;
use constant BIN_INDEX => 5;
use Getopt::Long;
use File::Basename;
use Cwd 'realpath';

#Used for stripping heap output
my $divider = "-----------------------------------------------------------------------";
my $labels = "=====     =====       ===   ==========                                       ====    ======";
my $nonObject = 'non-object';

#Used for stripping heap output
my $dividerOld = "-----------------------------------------------------------------------";
my $labelsOld = "==========  ====               ======     =====     =====       ===";


my $isOldFormat = 0;

my %objectClassList1;
my %objectClassList2;
my %objectClassList1_only;
my %objectClassList2_only;
my %objectClassListIntersection;

my $totalBytes1;
my $totalBytes2;
my $totalNodes1;
my $totalNodes2;

my @heapFileLines1;
my @heapFileLines2;

# command line arg vars
my $help;
my $output_type;
my $file_one_name;
my $file_two_name;
GetOptions(
    "h|help" => \$help,
    "f|format=s" => \$output_type,
    "o|file_one=s" => \$file_one_name,
    "t|file_two=s" => \$file_two_name,
    );

#handle help
if ($help) { 
	usage();
}
 
# gather required arguments
my $path1 = $ARGV[0];
my $path2 = $ARGV[1];
usage () if(!$path1 || !$path2);
if(! -e $path1) {
    print "file $path1 could not be found\n";
    usage();
}
if(! -e $path2) {
    print "file $path2 could not be found\n";
    usage();
}
my $fullpath1 = realpath($path1);
my $fullpath2 = realpath($path2);
if(!$file_one_name || $file_one_name eq "") {
    my $dir = dirname($path1);
    $file_one_name = basename($path1); 
}
if(!$file_two_name || $file_two_name eq "") {
    $file_two_name = basename($path2); 
}
# set reasonable defaults if none are passed in.
my $output_path = $ARGV[2];
$output_type = 'html' if(!$output_type);
$output_path = '/tmp/heapdiff.html' if(!$output_path && $output_type eq 'html');
$output_path = '/tmp/heapdiff.txt' if(!$output_path && $output_type ne 'html');

if ($output_type ne "html" && $output_type ne "tab" && $output_type ne "csv" && $output_type ne "formatted") {
	print "3rd argument should be html, tab, csv or formatted\n";
	usage();
}

# used for printing.
my @diff_header = ("Object", "Framework", "$file_one_name Count", "$file_two_name Count",  "Count Delta", "$file_one_name Total Memory  (B)", "$file_two_name Total Memory  (B)", "Total Memory Delta (B)", "Total Memory Ratio (x)", "$file_one_name Average Instance Size  (B)", "$file_two_name Average Instance Size  (B)",  "Average Size Delta (B)", "Average Size Ratio (x)", "% of Total Object Memory Regression");#  "Delta Normalized for Count");
our $delim = "\t";
our $row_format_string;
our $left_count = 1;
my ($file1, $file2, $out);
open($file1, "<$path1") or die "Cannot open $path1, $!";
open($file2, "<$path2") or die "Cannot open $path2, $!";

if($output_path ne "STDOUT") {
	open($out, ">$output_path") or die "Cannot open $output_path:$!";
	select $out;
}

#Strip data and add to hashmap
&stripHeapOutput($file1, \@heapFileLines1, \$totalBytes1, \$totalNodes1);
&addDataToHash(\@heapFileLines1, \%objectClassList1);

&stripHeapOutput($file2, \@heapFileLines2, \$totalBytes2, \$totalNodes2);
&addDataToHash(\@heapFileLines2, \%objectClassList2);


my @total_header = ("", "$file_one_name", "$file_two_name", "Delta", "Ratio");
if($output_type eq 'html') {
	print "<html><head>\n";
	while(<DATA>) { print $_; }
	print "</head><body>\n";
	print "<strong>Totals</strong><br/>\n";
	@total_header = ("", "<a href='file://$fullpath1'>$file_one_name</a>", "<a href='file://$fullpath2'>$file_two_name</a>", "Delta (B)", "Ratio (x)");
 	@diff_header =  ("<a href='#object'>Object</a>", "<a href='#framework'>Framework</a>", "<a href='#32_count'>$file_one_name Count</a>", "<a href='#64_count'>$file_two_name Count</a>",  "<a href='#count_delta'>Count Delta</a>", "<a href='#32_total'>$file_one_name Total Memory  (B)</a>", "<a href='#64_total'>$file_two_name Total Memory  (B)</a>", "<a href='#total_delta'>Total Memory Delta (B)</a>", "<a href='#total_ratio'>Total Memory Ratio (x)</a>", "<a href='#32_avg'>$file_one_name Average Instance Size  (B)</a>", "<a href='#64_avg'>$file_two_name Average Instance Size  (B)</a>",  "<a href='#avg_delta'>Average Size Delta (B)</a>", "<a href='#avg_ratio'>Average Size Ratio (x)</a>", "<a href='#per_regression'>% of Total Object Memory Regression</a>"); #,  "<a href='#norm'>Delta Normalized for Count</a>");
}
else {
	print "Totals\n";
	print "-------\n";
}
print_header(\@total_header, $output_type, 'totals');
my @byte_totals = ("Total Object Bytes");
my $byte_total_1 = getByteTotals(\%objectClassList1, $path1);
push(@byte_totals, $byte_total_1);
my $byte_total_2 = getByteTotals(\%objectClassList2, $path2);
push(@byte_totals, $byte_total_2);
my $byte_delta = $byte_total_2-$byte_total_1;
push(@byte_totals, $byte_delta);
my $byte_delta_per = sprintf("%.1f", $byte_total_2/$byte_total_1);
push(@byte_totals, $byte_delta_per);
print_row(\@byte_totals, $output_type);
#print "\nTotal Object Instance Count\t";
my @count_totals = ("Total Object Instance Count");
my $count_total_1 = getCountTotals(\%objectClassList1, $path1);
my $count_total_2 = getCountTotals(\%objectClassList2, $path2);
push(@count_totals, $count_total_1);
push(@count_totals, $count_total_2);
my $count_delta = $count_total_2-$count_total_1;
push(@count_totals, $count_delta);
my $count_delta_per = sprintf("%.1f", $count_total_2/$count_total_1);
push(@count_totals, $count_delta_per);
print_row(\@count_totals, $output_type);
my @mem_alloc = ("Total Memory Allocated");
push(@mem_alloc, $totalBytes1);
push(@mem_alloc, $totalBytes2);
push(@mem_alloc, $totalBytes2-$totalBytes1);
push(@mem_alloc, sprintf("%.1f", $totalBytes2/$totalBytes1));
print_row(\@mem_alloc, $output_type);
print_footer($output_type);

if($output_type ne 'html') {
	print "\n\n\n\n";
	print "$file_one_name-$file_two_name Differences\n";
	print "--------------------------\n";
}
else {
	print "<br/><br/>";
	print "<strong>$file_one_name-$file_two_name Differences</strong><br/>\n";
}

compareData();
analyzeIntersection();

my @left_headers_32 = ("Object", "Framework", "$file_one_name Count", "$file_one_name Total Memory", "$file_one_name Average Instance Size  (B)", "% of Memory Allocated");
if($output_type ne 'html') {
	print "\n\n\n\n";
	print "$file_one_name only objects\n";
	print "-----------------------\n";
}
else {
	print "<br/><br/>";
	print "<strong>$file_one_name only objects</strong><br/>\n";
	@left_headers_32 = ("<a href='#object'>Object</a>", "<a href='#framework'>Framework</a>", "<a href='#32_count'>$file_one_name Count</a>", "<a href='#32_total'>$file_one_name Total Memory  (B)</a>", "<a href='#32_avg'>$file_one_name Average Instance Size  (B)</a>", "<a href='#per_mem_alloc'>% of Memory Allocated</a>");
}

analyzeLeftovers(\%objectClassList1_only, \@left_headers_32, $totalBytes1);

my @left_headers_64 = ("Object", "Framework", "$file_two_name Count", "$file_two_name Total Memory", "$file_two_name Average Instance Size  (B)", "% of Memory Allocated");
if($output_type ne 'html') {
	print "\n\n\n\n";
	print "$file_two_name only objects\n";
	print "-----------------------\n";
}
else {
	print "<br/><br/>";
	print "<strong>$file_two_name only objects</strong><br/>\n";
	@left_headers_64 = ("<a href='#object'>Object</a>", "<a href='#framework'>Framework</a>", "<a href='#64_count'>$file_two_name Count</a>", "<a href='#64_total'>$file_two_name Total Memory  (B)</a>", "<a href='#64_avg'>$file_two_name Average Instance Size  (B)</a>", "<a href='#per_mem_alloc'>% of Memory Allocated</a>");
}

analyzeLeftovers(\%objectClassList2_only, \@left_headers_64, $totalBytes2);

if($output_type eq 'html') {
#  	@diff_header =  ("<a href='#object'>Object</a>", "<a href='#framework'>Framework</a>", "<a href='#32_count'>$file_one_name Count</a>", "<a href='#64_count'>$file_two_name Count</a>",  "<a href='#count_delta'>Count Delta</a>", "<a href='#32_total'>$file_one_name Total Memory  (B)</a>", "<a href='#64_total'>$file_two_name Total Memory  (B)</a>", "<a href='#total_delta'>Total Memory Delta (B)</a>", "<a href='#total_ratio'>Total Memory Ratio (x)</a>", "<a href='#32_avg'>$file_one_name Average Instance Size  (B)</a>", "<a href='#64_avg'>$file_two_name Average Instance Size  (B)</a>",  "<a href='#avg_delta'>Average Size Delta (B)</a>", "<a href='#avg_ratio'>Average Size Ratio (x)</a>", "<a href='#per_regression'>% of Total Object Memory Regression</a>",  "<a href='#norm'>Delta Normalized for Count</a>");
	my $explain =<<END_EXPLAIN;
	<br><br>
<strong>Legend</strong>&nbsp<span class='tiny'><a href='#top'>Back to Top</a></span>
<table id='legend'>
	<thead>
	<tr id='header'><th>Field Name</th><th>Field Description</th></tr>
	</thead>
	<tbody>
	<tr class='row_no_stripe'><td class='field_name'><a name='object'>Object</td><td class ='field_des'>The object class examined</td></tr>
	<tr class='row_stripe'><td class='field_name'><a name='framework'>Framework</td><td class ='field_des'>The framework or binary where this class is defined</td></tr>
	<tr class='row_no_stripe'><td class='field_name'><a name='32_count'>$file_one_name Count</td><td class ='field_des'>The number of instances of this class allocated by the $file_one_name version of the process</td></tr>
	<tr class='row_stripe'><td class='field_name'><a name='64_count'>$file_two_name Count</td><td class ='field_des'>The number of instances of this class allocated by the $file_two_name version of the process</td></tr>
	<tr class='row_no_stripe'><td class='field_name'><a name='count_delta'>Count Delta</td><td class ='field_des'>The difference ($file_two_name - $file_one_name) in number of instances of this class allocated in the $file_one_name and $file_two_name versions of the process.</td></tr>
	<tr class='row_stripe'><td class='field_name'><a name='32_total'>$file_one_name Total Memory</td><td class ='field_des'>The total memory allocated for instances of this class in the $file_one_name version of the process.  Measured in bytes.</td></tr>
	<tr class='row_no_stripe'><td class='field_name'><a name='64_total'>$file_two_name Total Memory (B)</td><td class ='field_des'>The total memory allocated for instances of this class in the $file_two_name version of the process.  Measured in bytes.</td></tr>
	<tr class='row_stripe'><td class='field_name'><a name='total_delta'>Total Memory Delta (B)</td><td class ='field_des'>The difference ($file_two_name - $file_one_name) in the total memory allocated for this class in the $file_one_name and $file_two_name versions of the process.</td></tr>
	<tr class='row_no_stripe'><td class='field_name'><a name='total_ratio'>Total Memory Ratio (x)</td><td class ='field_des'>The ratio ($file_two_name/$file_one_name) of total memory allocated for this class in $file_two_name vs. the total memory allocated in $file_one_name.</td></tr>
	<tr class='row_stripe'><td class='field_name'><a name='32_avg'>$file_one_name Average Instance Size  (B)</td><td class ='field_des'>The average instance size in bytes for this object in the $file_one_name process.  This is calculated by dividing total memory by number of instances.</td></tr>
	<tr class='row_no_stripe'><td class='field_name'><a name='64_avg'>$file_two_name Average Instance Size  (B)</td><td class ='field_des'>The average instance size in bytes for this object in the $file_two_name process.  This is calculated by dividing total memory by number of instances.</td></tr>
	<tr class='row_stripe'><td class='field_name'><a name='avg_delta'>Average Size Delta (B)</td><td class ='field_des'>The difference ($file_two_name - $file_one_name) in average instance size for this class measured in bytes.</td></tr>
	<tr class='row_no_stripe'><td class='field_name'><a name='avg_ratio'>Average Size Ratio (x)</td><td class ='field_des'>The ratio ($file_two_name/$file_one_name) of average instance size for this class.</td></tr>
	<tr class='row_stripe'><td class='field_name'><a name='per_regression'>% of Total Object Memory Regression</td><td class ='field_des'>The percent of the increase in memory allocations for objects accounted for by this object class.  The larger the percentage listed, the larger the effect this object has your process's memory footprint. </td></tr>
<!-- 	<tr class='row_no_stripe'><td class='field_name'><a name='norm'>Delta Normalized for Count</td><td class ='field_des'>The difference in memory ($file_two_name - $file_one_name) that would have been observed if the object count in $file_two_name was equal to the object count in $file_one_name.  This is a measure of object bloat normalizing for differences in instance count.</td></tr> -->
	<tr class='row_no_stripe'><td class='field_name'><a name='per_mem_alloc'>% of Total Memory Allocated</td><td class ='field_des'>The percent of the total memory allocated for classes found in the $file_one_name/$file_two_name version of the process and not the $file_two_name/$file_one_name version</td></tr>	
	<tr class='row_stripe'><td class='field_name'><a name='tot_obj_bytes'>Total Object Bytes</td><td class='field_des'>The total number of bytes allocated for objects in the specified architecture of the process. Measured in bytes.</td></tr>	
	<tr class='row_no_stripe'><td class='field_name'><a name='tot_obj_count'>Total Object Instance Count</td><td class ='field_des'>The total number of objects allocated in the specified architecture of the process.</td></tr>	
	<tr class='row_stripe'><td class='field_name'><a name='tot_mem_alloc'>Total Memory Allocated</td><td class ='field_des'>The total amount of memory allocated by the specified architecture of process.  Measured in bytes.</td></tr>	
	
	</tbody>
</table>
END_EXPLAIN
	print $explain;
	print "<script language='javascript'>\n";
	print "\tvar tables = findTables();\n";
	print "\taddTableSorting(tables);\n";
	print "\tsort_table('diff', 13);\n";
	print "\tsort_table('totals', 2);\n";
	print "\tsort_table('left_1', 5);\n";
	print "\tsort_table('left_2', 5);\n";
	print "</script>\n";
	print "</body></html>\n";
}

if($output_type eq 'html') {
	exec 'open', '-a', '/Applications/Safari.app', $output_path;
}
else {
	print STDOUT "Your comparison file is available at $output_path\n";
	exec 'open', $output_path;
}

sub round {
	my $number = shift;
	return int($number + .5 * ($number <=> 0));
}
sub commify {
   my $input = shift;
   $input = reverse $input;
   $input =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
   return reverse $input;
}
sub stripHeapOutput{

    my $reachedData = 0;
    my ($FILE, $arrayRef, $totalBytesRef, $totalNodesRef) = @_;
    
    
    
    foreach (<$FILE>){
	    #Skip newline lines
	    if(length() <= 1 || $_ =~/$nonObject/){
		    next;
	    }
	    #Scan for the label underlines, ignoring everything until they are found
	    if(!$reachedData){
		    if ($_ =~/$labels/){
			$reachedData = 1;
			$isOldFormat = 0;
		    }
		    elsif ($_ =~ /$labelsOld/){
		 	$reachedData = 1;
			$isOldFormat = 1;	
		    }
		    
		    #Check to see if this is a total allocation line
		    if ($_ =~/Zone [\w ]+: Overall size: \w+; (\d+) nodes malloced for (\d+)KB/){
			$$totalNodesRef += $1;
			$$totalBytesRef += $2;
		    }	
	    }
	    #Once they are found, print the valid lines.
	    else{
		    if($_=~/$divider/ || $_=~/$dividerOld/){
			    $reachedData = 0;
		    }
		    else{
			push @$arrayRef, $_;   
		    }
	    }
    }

    #Convert to bytes
    $$totalBytesRef *= 1024;
}

sub getByteTotals{
	
	my $hash_ref = shift @_;
	my $path = shift @_;
	my $bytes;
	my $count;
	foreach (keys %$hash_ref){
		my $ref = $hash_ref->{$_};
		$bytes += $ref->[BYTES_INDEX];
	}
	return $bytes;
}

sub getCountTotals{
	
	my $hash_ref = shift @_;
	my $path = shift @_;
	my $bytes;
	my $count;
	foreach (keys %$hash_ref){
		my $ref = $hash_ref->{$_};
		$count += $ref->[COUNT_INDEX];	
	}
	return $count;
}


sub analyzeIntersection{
	
	print_header(\@diff_header, $output_type, 'diff');
# 	print "Object\tFramework\t$file_one_name Instance Count\t$file_two_name Instance Count\t$file_one_name Total Bytes for Instances\t$file_two_name Total Bytes for Instances\t$file_one_name Average Instance Size (Bytes)\t$file_two_name Average Instance Size (Bytes)\tCount Differences\tTotal Memory Difference (Bytes)\tAverage Growth\tNormalizedGrowth\n";

	foreach (keys %objectClassListIntersection){
		my $array1_ref = $objectClassList1{$_}; 
		my $array2_ref = $objectClassList2{$_};
		my @array1 = @$array1_ref;
		my @array2 = @$array2_ref;
		my $avg1 = round($array1[BYTES_INDEX]/$array1[COUNT_INDEX]);
		my $avg2 = round($array2[BYTES_INDEX]/$array2[COUNT_INDEX]);
		my $avg_diff = $avg2 - $avg1;
		my $count_diff = $array2[COUNT_INDEX]-$array1[COUNT_INDEX];
		my $bytes_diff = $array2[BYTES_INDEX]-$array1[BYTES_INDEX];
		my $bytes_ratio = sprintf("%.2f", $array2[BYTES_INDEX]/$array1[BYTES_INDEX]);
		my $per_obj_regression = sprintf("%.1f", $bytes_diff/$byte_delta*100);
		my $avg_ratio = sprintf("%.2f",$avg2/$avg1);
#		my $normGrowth = sprintf("%.2f", $array1[BYTES_INDEX] * $avg_ratio);
		my @results = ($array1[OBJ_INDEX], $array1[BIN_INDEX], $array1[COUNT_INDEX], $array2[COUNT_INDEX], $count_diff, $array1[BYTES_INDEX], $array2[BYTES_INDEX], $bytes_diff, $bytes_ratio, $avg1, $avg2, $avg_diff, $avg_ratio, $per_obj_regression); #, $normGrowth);
		print_row(\@results, $output_type);
		#print $array1[OBJ_INDEX]."\t".$array1[BIN_INDEX]."\t".$array1[COUNT_INDEX]."\t".$array2[COUNT_INDEX]."\t".$array1[BYTES_INDEX]."\t".$array2[BYTES_INDEX]."\t".$avg1."\t".$avg2."\t".$cdiff."\t".$bdiff."\t".$growth."\t".$normGrowth."\n";
	}
	print_footer($output_type);
}

sub analyzeLeftovers {
	my ($dict, $header, $total_mem) = @_;

	print_header($header, $output_type, "left_$left_count");
	$left_count += 1;
	foreach my $key (keys %$dict) {
		my $ref = $dict->{$key};

		my $avg1 = round($ref->[BYTES_INDEX]/$ref->[COUNT_INDEX]);
		my $per_mem_allocated = sprintf("%.1f", $ref->[BYTES_INDEX]/$total_mem*100);
		my @results = ($ref->[OBJ_INDEX], $ref->[BIN_INDEX], $ref->[COUNT_INDEX], $ref->[BYTES_INDEX], $avg1, $per_mem_allocated);
		print_row(\@results, $output_type);
	}
	print_footer($output_type);
}
sub htmlencode_array {
	my $arr_ref = shift;
	foreach my $mem (@$arr_ref) {
		$mem = htmlencode($mem);
		$mem =~ s/\-0.0/0.0/g;
	}
}
sub htmlencode {
	my $str = shift;
	$str =~ s/</\&lt\;/g;
	$str =~ s/>/\&gt\;/g;
	return $str;
}
sub print_footer {
	my $output_type = shift;
	if($output_type eq "html") {
		print "</tbody></table>\n";
	}
}
sub print_header {
	my ($row_headings, $output_type, $table_name) = @_;
	
	if($output_type eq 'csv' || $output_type eq 'tab' || $output_type eq 'html') {
		if ($output_type eq 'csv' ) {
			$delim = ",";
		}
		if ($output_type eq 'html') {
			print "<table id='$table_name'>\n\t<thead><tr id='header'><th>";
			$delim = "</th><th>";
		}
		print join($delim, @$row_headings); 
		if ($output_type eq 'html') {	
			$delim = "</td><td>";
			print "</th></tr></thead><tbody>\n";
		}
		else {
			print "\n";
		}
	}
	elsif ($output_type eq 'formatted') {
		$format_string = "";
		$row_format_string = "";
		foreach my $field (@$row_headings) {
			my $len = (length $field)+2;
			my $char_code = 'g';
			if($field eq "Object") {
				$char_code = 's';
				$len = 35;
			}
			if($field eq "Framework") {
				$len = 20;
				$char_code = 's';
			}
			if($field eq "File") {
				$len = 30;
				$char_code = 's';
			}
			if($field =~/rowth/) {
				$char_code = ".2f";
			}
                        if($field eq "") {
                                $char_code = "30s";
                        }
			$row_format_string .= "%-$len$char_code";
			$format_string .= "%-" . $len ."s";
		}
		$row_format_string .= "\n";
		$format_string .="\n";
#		print "row_format_string is $row_format_string\n";
		printf($format_string, @$row_headings);	
	}
}
our $stripe = 0;
sub print_row {
	my ($row, $output_type) = @_;
		if($output_type eq 'csv' || $output_type eq 'tab' || $output_type eq 'html') {
		if ($output_type eq 'html') {
			htmlencode_array($row);
			my $class = "class='row_no_stripe'";
			if($stripe) {
				$class = "class='row_stripe'";
			}
			print "\t<tr id='" . $row->[0] . "' $class><td>";
			$stripe = !$stripe;
		}
		print join($delim, @$row); 
		if ($output_type eq 'html') {	
			print "</td></tr>\n";
		}
		else {
			print "\n";
		}
	}
	elsif ($output_type eq 'formatted') {
		printf($row_format_string, @$row);	
	}
}
sub compareData{

	
	foreach my $key (keys %objectClassList1){
		local $objectClassData = $objectClassList2{$key};
		#In objectClassListIntersection
		if($objectClassData){
			my @pair = ($objectClassList1{$key}, $objectClassData);
			$objectClassListIntersection{$key} = @pair;
		}
		#In just objectClassList1
		else{
			$objectClassList1_only{$key} = $objectClassList1{$key};
		}
	}
	
	foreach my $key (keys %objectClassList2){
		local $objectClassData = $objectClassList1{$key};
		#In objectClassListIntersection
		if(!$objectClassData){
			$objectClassList2_only{$key} = $objectClassList2{$key};
		}
	}
}

#FD %hashToAdd
sub addDataToHash{
	my ($arrayRef, $hashRef)= @_;
	
	
	foreach (@$arrayRef){
		#Skip comments
		if($_=~/^\s*\#/){
			next;
		}
		my @columns = split;
		my $numColumns = @columns;
		
		if($isOldFormat){
		    my $classColumn;
		    if($numColumns > 6){
			    for($i=0; $i<$numColumns-5; $i++){
				    $classColumn .= $columns[$i];
			    }	    
		    }
		    else{
		    	$classColumn = $columns[0];
		    }
		    @columns = ($columns[$numColumns-3], $columns[$numColumns-2], $columns[$numColumns-1], $classColumn, $columns[$numColumns-5], $columns[$numColumns-4]);
		}
		else{
		    if($numColumns > 6){
			    my $thirdColumn;
			    for($i=3; $i<$numColumns-2; $i++){
				    $thirdColumn .= " ".$columns[$i];
			    }
			    @columns = ($columns[0], $columns[1],$columns[2], $thirdColumn, $columns[$numColumns-2], $columns[$numColumns-1]);
		    }
		    if($numColumns < 6){
			    for($j=0; $j<(6-$numColumns); $j++){
				    push @columns, " ";
			    }
		    }
		}
				
		#Check to see if new object
		my $ref = $hashRef->{'CLASS:'.$columns[OBJ_INDEX].' BINARY:'.$columns[BIN_INDEX]};
		#Potentially new object
		if($ref){	
			#Compare binary
			#Update the values (we don't care about average)
			$ref->[COUNT_INDEX] += $columns[COUNT_INDEX];
			$ref->[BYTES_INDEX] += $columns[BYTES_INDEX];

		}
		#New object
		else{
			$hashRef->{'CLASS:'.$columns[OBJ_INDEX].' BINARY:'.$columns[BIN_INDEX]} = \@columns;
		}
	}	
}


sub usage{
	print "heapdiff.pl <inputFile1> <inputFile2> [<outputFile>] [-f|format html | tab | formatted | csv] [-h|help] [-o|file_one <Label1>] [-t|file_two <Label2>]\n";
	print "\n";
	print "Options:\n";
	print "\t-h | -help\tPrints this message.\n";
	print "\t-f | -format\tSelects the format of the output. Options are html,\n\t\t\ttab, formatted, csv\n";
	print "\t-o | -file_one <Label1>\tAllows the user to change the display name of the\n\t\t\tdata from the first file to <Label1>, otherwise the file name will be used.\n";
	print "\t-t | -file_two <Label2>\tAllows the user to change the display name of the\n\t\t\tdata from the second file to <Label2>, otherwise the file name will be used.\n";
	print "\t<outputFile>\tPath to an alternate output file. The default is\n\t\t\t/tmp/heapdiff.html or heapdiff.txt.\n";
	exit;
}

__DATA__
<script language="javascript">
var all_tables = new Array();
function findTables() {
	 var tables = document.getElementsByTagName("table");
	 if(tables.length == 0 ) {
 		  alert("No tables found");
	 }
	 return tables;
}
function addTableSorting(tables) {
	 //tables is a NodeList
	 //  first we add the sorting, then we can go build our data structures for later use
	 // this will make the js appear fast.
	 for(var i = 0; i < tables.length; i++) {
		 var table = tables[i];
		 var table_header = table.childNodes[1];
		 addHeaderSortIcons(table_header, table.attributes['id'].nodeValue);
	 }
	 for(var i = 0; i< tables.length; i++) {
	 	 var table = tables[i];
	 	 parseTable(table, table.attributes['id'].nodeValue);
	 }
}
function parseTable(table, table_name) {
	 var tbodies = table.getElementsByTagName('tbody');
	 var tbody = tbodies[0];
	 var rows = tbody.getElementsByTagName('tr');
	 var table_data = new Array();
	 for (var i = 0; i<rows.length; i++) {
	     var row = rows[i];
	     var row_name = row.attributes['id'];
	     var columns = row.getElementsByTagName('td');
	     var row_data = new Array();
	     for (var j = 0; j< columns.length; j++) {
	     	 var td = columns[j];
		 var datum = td.firstChild.nodeValue;
		 if(td.childNodes.length > 1) {
		 	datum = td.childNodes[1].nodeValue;
		 }
		if(!datum) {
			datum = "";
		}
		 row_data.push(datum);
	     }
	     table_data.push(row_data);
	 }
	 all_tables[table_name] = table_data;
}
function addHeaderSortIcons(header, table_name) {
	 // header is a Node
	 header.setAttribute('class', 'row_heading');
	 var ths = header.getElementsByTagName('th');
	 for(var i = 0; i< ths.length; i++) {
	 	 var td = ths[i];
		 td.appendChild(document.createTextNode(" "));
		 var a = document.createElement('a');
		 a.setAttribute('href', "javascript:sort_table('" + table_name + "', " + i + ");");
		 var img = document.createElement('img');
		 img.setAttribute('src', 'file:///usr/share/heapdiff/DescendingArrow.png');
		 img.setAttribute('id', table_name + "_sort_" + i);
		 a.appendChild(img);
		 td.appendChild(a);
	 }
}
function sort_table(table_name, row) {
	 var img = document.getElementById(table_name+"_sort_"+row);
	 if(img.src == 'file:///usr/share/heapdiff/DescendingArrow.png') {
	 	    dir = 0;
		    img.src = 'file:///usr/share/heapdiff/AscendingArrow.png';
	 }
	 else {
	      dir = 1;
       	      img.src = 'file:///usr/share/heapdiff/DescendingArrow.png';
	 }
	 var table = document.getElementById(table_name);
	 var table_data = all_tables[table_name];
	 if (!table) {
	    alert ("couldn't find the table element in the page for "+table_name);
	 }
	 if(!table_data) {
 	    alert ("couldn't find the table element in the data for "+table_name);
         }
	 if(!table || !table_data) {
	    return;
	 }
	 col = row;
	 table_data.sort(order);
	 erase_table(table_name);
	 print_table(table_name);	 
}
function addCommas(nStr)
{
	nStr += '';
	x = nStr.split('.');
	x1 = x[0];
	x2 = x.length > 1 ? '.' + x[1] : '';
	var rgx = /(\d+)(\d{3})/;
	while (rgx.test(x1)) {
		x1 = x1.replace(rgx, '$1' + ',' + '$2');
	}
	return x1 + x2;
}
function print_table(table_name) {
	 var table = document.getElementById(table_name);
	 var table_data = all_tables[table_name];
	 var stripe = false;
	 var tbody = document.createElement('tbody');
	 table.appendChild(tbody);
	 for (var i = 0; i< table_data.length; i++) {
	     var row = table_data[i];
	     var tr = document.createElement('tr');
	     tr.setAttribute('id', row[0]);
	     tbody.appendChild(tr);
	     if(stripe) {
	     	tr.setAttribute('class', 'row_stripe');
	     }
	     else {
	     	  tr.setAttribute('class', 'row_no_stripe');
	     }
	     stripe = !stripe;
	     for (var j = 0; j<row.length; j++) {
	     	 var datum = row[j];
		 var td = document.createElement('td');
		 tr.appendChild(td);
		 td.appendChild(document.createTextNode(addCommas(datum)));
	     }
	 }
}
// dirty hack for sort. I should have probably overridden sort() for Array, but 
// I didn't have time. so we have this instead. whenever you call sort on tables,
// you have to set the column and direction.
var col = 0;
var dir = 0;
function order(a, b) {
	 if( dir == 0 ) {
	 	 if( col == 0  || col == 1) { return b[col].localeCompare(a[col]); }
	 	 else { return b[col] - a[col]; }
	 }
	 else {
	 	 if( col == 0 || col == 1) { return a[col].localeCompare(b[col]); }
	 	 else { return a[col]-0 - b[col]; }
	 }
}
function erase_table(name) {
	 var table = document.getElementById(name);
	 table.removeChild(table.lastChild);
}
</script>
<style type="text/css">
BODY {font-family: Helvetica, sans-serif; }
TABLE { background-color: #666666; border-spacing: 1px; }
TH { font-family: Helvetica, sans-serif; font-size: small; text-align: right; padding: 2px 6px 2px 6px; }
TD { font-family: Helvetica, sans-serif; font-size: small; text-align: right; white-space: nowrap; padding: 2px 6px 2px 6px; }
*.field_name { text-align:left }
*.field_des { text-align: left }
*.name { text-align: left; font-weight: bold; }
*.row_heading { background-color: #f4f4f4 }
*.row_no_stripe { background-color: #ffffff }
*.row_stripe { background-color: #edf3fe }
*.tiny { font-size: xx-small }
</style>
