<?xml version="1.0" encoding="us-ascii"?>
<!--
	Upgrade old sdefs to the current format; see sdef(5) History for details.
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

	<xsl:output
		method="xml"
		doctype-system="file://localhost/System/Library/DTDs/sdef.dtd"
		indent="yes"
	/>

	<!-- By default, copy everything through. -->
	<xsl:template match="comment() | processing-instruction() | *">
		<xsl:copy>
			<xsl:apply-templates select="@*[not(name()='type')] | @type[not(contains(., '|') or contains(., 'list of '))]"/>
			<xsl:apply-templates select="@type"/>
			<!-- must handle complex type tranformation *after* all other attributes,
			     because it might generate element nodes. -->
			<xsl:apply-templates select="child::node()"/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="@*">
		<xsl:copy/>
	</xsl:template>

	<!--
	  10.4 to 10.5 changes.
	  -->

	<!-- responds-to "name" attribute is now "command". -->
	<xsl:template match="responds-to/@name">
		<xsl:attribute name="command"><xsl:value-of select="."/></xsl:attribute>
	</xsl:template>


	<!--
	  10.3 to 10.4 changes.
	  -->

	<!-- Hoist contents of all collector elements. -->
	<xsl:template match="types | classes | commands | events | properties | elements | synonyms | responds-to-commands | responds-to-events">
		<xsl:apply-templates select="child::node()"/>
	</xsl:template>

	<!-- Transmogrify "method" attributes into "key". -->
	<xsl:template match="property/cocoa/@method | element/cocoa/@method">
		<xsl:attribute name="key"><xsl:value-of select="."/></xsl:attribute>
	</xsl:template>

	<!-- Transmogrify boolean attributes to "yes" or "no". -->
	<xsl:template match="@hidden['hidden'] | @optional['optional']">
		<xsl:attribute name="{name()}">yes</xsl:attribute>
	</xsl:template>

	<!-- Reverse sense of "not-in-properties". -->
	<xsl:template match="@not-in-properties['not-in-properties']">
		<xsl:attribute name="in-properties" select='"no"'/>
	</xsl:template>

	<!-- Transmogrify complex "type" attributes into "type" elements. -->
	<xsl:template match="@type">
		<xsl:choose>
			<xsl:when test="contains(., '|') or contains(., 'list of ')">
				<xsl:call-template name="munge-type-attribute">
					<xsl:with-param name="a" select="."/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="munge-simple-type">
					<xsl:with-param name="a" select="."/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="munge-type-attribute">
		<xsl:param name="a"/>
		<xsl:choose>
			<xsl:when test="contains($a, '|')">
				<xsl:call-template name="munge-type-attribute">
					<xsl:with-param name="a" select="normalize-space(substring-before($a, '|'))"/>
				</xsl:call-template>
				<xsl:call-template name="munge-type-attribute">
					<xsl:with-param name="a" select="normalize-space(substring-after($a, '|'))"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="contains($a, 'list of ')">
				<type>
					<xsl:call-template name="munge-simple-type">
						<xsl:with-param name="a" select="normalize-space(substring-after($a, 'list of '))"/>
					</xsl:call-template>
					<xsl:attribute name="list">yes</xsl:attribute>
				</type>
			</xsl:when>
			<xsl:otherwise>
				<type>
					<xsl:call-template name="munge-simple-type">
						<xsl:with-param name="a" select="$a"/>
					</xsl:call-template>
				</type>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="munge-simple-type">
		<xsl:param name="a"/>
		<xsl:attribute name="type">
		<xsl:choose>
			<xsl:when test="$a='string'"><xsl:text>text</xsl:text></xsl:when>
			<xsl:when test="$a='object'"><xsl:text>specifier</xsl:text></xsl:when>
			<xsl:when test="$a='location'"><xsl:text>location specifier</xsl:text></xsl:when>
			<xsl:otherwise><xsl:value-of select="$a"/></xsl:otherwise>
		</xsl:choose>
		</xsl:attribute>
	</xsl:template>

</xsl:stylesheet>
